==> RSTouchscreenNG-0.94.3.zip: RSTouchscreenNG/projects/binding/src/lib/reducer/PrinterReducer.ts <==
/* (C) 2016-2020 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {
  PrinterState,
  INITIAL_PRINTER_STATE,
  GCodeFileState,
  TemperatureDataState,
  ToolState,
  GroupState,
  LogState
} from "../state/PrinterState";
import {PrinterAction} from "../actions/PrinterAction";
import {RSUtils} from "../RSUtils";

export function PrinterReducer(state: PrinterState = INITIAL_PRINTER_STATE, action: any): PrinterState {
  function orderBy(list: any[], field: string, reverse: boolean) {
    return list.sort(function (a: any[], b: any[]) {
      // ['-online', '-active', 'name']);//r;
      if (reverse) {
        if (a[field] < b[field]) {
          return -1;
        }
        if (a[field] > b[field]) {
          return 1;
        }
      } else {
        if (a[field] < b[field]) {
          return 1;
        }
        if (a[field] > b[field]) {
          return -1;
        }
      }
      return 0;
    });
  }

  function sortModels(pstate: PrinterState, allModels: GCodeFileState[]): GCodeFileState[] {
    let models: GCodeFileState[] = [];
    // tslint:disable-next-line
    const reverse = (pstate.sortBy & 1) === 1;
    // tslint:disable-next-line
    switch (pstate.sortBy & 254) {
      case 0:
        models = orderBy(allModels, "name", reverse);
        break;
      case 2:
        models = orderBy(allModels, "created", reverse);
        break;
      case 4:
        models = orderBy(allModels, "printTime", reverse);
        break;
      case 6:
        models = orderBy(allModels, "length", reverse);
        break;
      case 8:
        models = orderBy(allModels, "lines", reverse);
        break;
      case 10:
        models = orderBy(allModels, "filamentTotal", reverse);
        break;
      case 12:
        models = orderBy(allModels, "layer", reverse);
        break;
    }
    if (pstate.modelFilter !== "") {
      const filtered = [];
      for (const v of models) {
        if (v.name.indexOf(pstate.modelFilter) >= 0) {
          filtered.push(v);
        }
      }
      models = filtered;
    }
    if (pstate.activeGroupId !== "*") {
      const filtered = [];
      for (const v of models) {
        if (v.group === pstate.activeGroupId) {
          filtered.push(v);
        }
      }
      models = filtered;
    }
    return models;
  }

  function appendTemperatureEntry(list: TemperatureDataState[], temp: TemperatureDataState): TemperatureDataState[] {
    let start = 0;
    while (list.length > start && list[start].time + 300000 < temp.time) {
      start++;
    }
    return list.slice(start).concat(temp);
  }

  const parts: string[] = action.type.split("/");
  if (parts.length > 1) { // aggregated type with pointer

  } else {
    switch (action.type) {
      case PrinterAction.UPDATE_LOG_STATUS: {
        return Object.assign({}, state, {log: Object.assign({}, state.log, action.payload)});
      }
      case PrinterAction.ADD_LOG_LINE: {
        const log: LogState = Object.assign({}, state.log, {lastLogStart: action.payload.id});
        if (!state.log.paused) {
          const lineText = action.payload.text.replace(/[<>]/ig, function (m) {
            return "&" + (m === ">" ? "g" : "l") + "t;";
          });
          const newline = action.payload.time + ": " + lineText;
          if (log.lastText !== lineText) {
            log.history = log.history.slice(0);
            log.history.push("<div class=\"log" + action.payload.type + "\">" + newline + "</div>");
            log.history.splice(0, log.history.length - log.maxLogLines);
            log.repeat = 1;
            log.lastText = lineText;
          } else { // Repeated lines
            log.repeat++;
            log.history = log.history.slice(0);
            log.history[log.history.length - 1] = "<div class=\"log" + action.payload.type + "\">" + newline + " (" + log.repeat + ")</div>";
          }
        }
        return Object.assign({}, state, {log: log});
      }
      case PrinterAction.SET_CONFIG: {
        const extra = {config: action.payload.config};
        if (extra.config.extruders.length > state.activeExtruderId) {
          extra["retractSpeed"] = extra.config.extruders[state.activeExtruderId].retractSpeed;
          extra["extrudeSpeed"] = extra.config.extruders[state.activeExtruderId].extrudeSpeed;
        }
        return Object.assign({}, state, extra);
      }
      case PrinterAction.SET_WEBCAM_CONFIG: {
        const extra = {config: Object.assign({}, state.config, {webcams: action.payload.webcams})};
        return Object.assign({}, state, extra);
      }
      case PrinterAction.RESET_IMAGE_REFERENCE:
        return Object.assign({}, state, {imageUrls: {}});
      case PrinterAction.CLEAR_EEPROM:
        return Object.assign({}, state, {eeprom: [], eepromErrors: false});
      case PrinterAction.COPY_EEPROM_VALUE_TO_ORIG: {
        const epr = RSUtils.copyArray(state.eeprom);
        for (const v of epr) {
          v.valueOrig = v.value;
          v.class = "secondary outline";
        }
        return Object.assign({}, state, {eeprom: epr, eepromErrors: false});
      }
      case PrinterAction.SET_NEW_EEPROM_VALUE: {
        const epr = RSUtils.copyArray(state.eeprom);
        for (const v of epr) {
          if (v.pos === action.payload.pos) {
            v.value = action.payload.value;
          }
        }
        let okAll = true;
        for (const v of epr) {
          const r = /^-?\d+$/;
          const f = /^-?\d+\.?\d*$/;
          let val;
          let ok = true;
          if (v.type === "0") { // byte
            if (r.exec(v.value) === null) {
              ok = false;
            } else {
              val = parseInt(v.value, 10);
              ok = (val > -128 && val < 256);
            }
          }
          if (v.type === "1") { // word
            if (r.exec(v.value) === null) {
              ok = false;
            } else {
              val = parseInt(v.value, 10);
              ok = (val > -32768 && val < 65536);
            }
          }
          if (v.type === "2") { // long
            if (r.exec(v.value) === null) {
              ok = false;
            } else {
              val = parseInt(v.value, 10);
              ok = (val > -2147483648 && val < 4294967296);
            }
          }
          if (v.type === "3") { // float
            if (f.exec(v.value) === null) {
              ok = false;
            }
          }
          if (ok) {
            if (v.value === v.valueOrig) {
              v.class = "secondary outline";
            } else {
              v.class = "success";
            }
          } else {
            v.class = "error";
          }
          okAll = okAll && ok;
        }
        return Object.assign({}, state, {eeprom: epr, eepromErrors: !okAll});
      }
      case PrinterAction.APPEND_EEPROM: {
        const epr = RSUtils.copyArray(state.eeprom);
        for (const pl of action.payload) {
          const text = pl.text;
          let textNU = "";
          let unit = "";
          const p = text.indexOf("[");
          if (p >= 0) {
            textNU = text.substr(0, p);
          } else {
            textNU = text;
          }
          if (p >= 0) {
            const p2 = text.indexOf("]", p);
            if (p2 >= 0) {
              unit = text.substring(p + 1, p2).replace("^2", "²").replace("^3", "³");
            }
          }
          const x = Object.assign({}, pl, {textNU: textNU, unit: unit, class: "secondary outline"});
          epr.push(x);
        }
        return Object.assign({}, state, {eeprom: epr});
      }
      case PrinterAction.SET_RECOVER:
        return Object.assign({}, state, {recover: action.payload});
      case PrinterAction.UPDATE_SCRIPT: {
        const extra = {};
        extra[action.payload.name] = action.payload.script;
        return Object.assign({}, state, {scripts: Object.assign({}, state.scripts, extra)});
      }
      case PrinterAction.SET_GROUPS: {
        const newGroups: GroupState[] = [];
        const baseGroups: GroupState[] = [];
        const baseGroups2: GroupState[] = [];
        for (const x in action.payload.groups) {
          if (action.payload.groups.hasOwnProperty(x)) {
            const g = action.payload.groups[x];
            if (g.id === "#" || g.id === "*") {
              baseGroups.push(g);
              if (g.id === "#") {
                baseGroups2.push(g);
              }
            } else {
              newGroups.push(action.payload.groups[x]);
            }
          }
        }
        return Object.assign({}, state, {
          groups: action.payload.groups,
          activeGroupId: action.payload.activeGroup,
          groupsSorted: baseGroups.concat(orderBy(newGroups, "name", true)),
          groupsSortedNoAll: baseGroups2.concat(orderBy(newGroups, "name", true))
        });
      }
      case PrinterAction.UPDATE_STATE: {
        const newState = Object.assign({}, state, action.payload.state);
        if (state.activeGroupId !== newState.activeGroupId) {
          newState.modelsFiltered = sortModels(newState, newState.allModels);
        }
        return newState;
      }
      case PrinterAction.UPDATE_STATUS:
        return Object.assign({}, state, action.payload.status);
      case PrinterAction.SET_IMG_URL: {
        const a: { [key: string]: string } = {};
        a[action.payload.key] = action.payload.url;
        return Object.assign({}, state, {imageUrls: Object.assign({}, state.imageUrls, a)});
      }
      case PrinterAction.UPDATE_IMG_URLS:
        return Object.assign({}, state, {imageUrls: Object.assign({}, state.imageUrls, action.payload)});
      case PrinterAction.SET_MODELS: {
        return Object.assign({}, state, {
          allModels: action.payload,
          modelsFiltered: sortModels(state, action.payload)
        });
      }
      case PrinterAction.SET_SORT_BY: {
        return Object.assign({}, state, {sortBy: action.payload});
      }
      case PrinterAction.SORT_MODELS: {
        return Object.assign({}, state, {modelsFiltered: sortModels(state, state.allModels)});
      }
      case PrinterAction.SET_PRINTQUEUE:
        return Object.assign({}, state, {queue: action.payload});
      case PrinterAction.ADD_BED_TEMPERATURE: {
        const newBed: ToolState[] = RSUtils.copyArray(state.heatedBeds);
        newBed[0].tempHistory = appendTemperatureEntry(newBed[action.payload.id].tempHistory, action.payload.temp);
        return Object.assign({}, state, {heatedBeds: newBed});
      }
      case PrinterAction.ADD_CHAMBER_TEMPERATURE: {
        const newChamber: ToolState[] = RSUtils.copyArray(state.heatedChambers);
        newChamber[0].tempHistory = appendTemperatureEntry(newChamber[action.payload.id].tempHistory, action.payload.temp);
        return Object.assign({}, state, {heatedChambers: newChamber});
      }
      case PrinterAction.ADD_EXTRUDER_TEMPERATURE: {
        const newExtruders: ToolState[] = RSUtils.copyArray(state.extruders);
        newExtruders[action.payload.extruderId].tempHistory = appendTemperatureEntry(newExtruders[action.payload.extruderId].tempHistory, action.payload.temp);
        return Object.assign({}, state, {extruders: newExtruders});
      }
      case PrinterAction.UPDATE_GCODE: {
        const g: GCodeFileState = action.payload;
        let idx = -1;
        const newState: PrinterState = Object.assign({}, state);
        if (g.list === "models") {
          for (let i = 0; i < newState.allModels.length; i++) {
            if (newState.allModels[i].id === g.id) {
              idx = i;
              break;
            }
          }
          if (idx !== -1) {
            newState.allModels = newState.allModels.slice(0, idx).concat([g], newState.allModels.slice(idx + 1));
            newState.modelsFiltered = sortModels(newState, newState.allModels);
          }
        } else {
          for (let i = 0; i < newState.queue.length; i++) {
            if (newState.queue[i].id === g.id) {
              idx = i;
              break;
            }
          }
          if (idx !== -1) {
            newState.queue = newState.queue.slice(0, idx).concat([g], newState.queue.slice(idx + 1));
          }
        }
        return newState;
      }
      default:
        return state;
    }
  }
}

==> RSTouchscreenNG-1.3.0.zip: RSTouchscreenNG/projects/binding/src/lib/reducer/PrinterReducer.ts <==
/* (C) 2016-2021 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {
  PrinterState,
  INITIAL_PRINTER_STATE,
  GCodeFileState,
  TemperatureDataState,
  ToolState,
  GroupState,
  LogState
} from "../state/PrinterState";
import {PrinterAction} from "../actions/PrinterAction";
import {RSUtils} from "../RSUtils";

const testTempResponse = /\b[TB]\d*:\s*[\d-]/;

export function PrinterReducer(state: PrinterState = INITIAL_PRINTER_STATE, action: any): PrinterState {
  function orderBy(list: any[], field: string, reverse: boolean) {
    return list.sort(function (a: any[], b: any[]) {
      // ['-online', '-active', 'name']);//r;
      if (reverse) {
        if (a[field] < b[field]) {
          return -1;
        }
        if (a[field] > b[field]) {
          return 1;
        }
      } else {
        if (a[field] < b[field]) {
          return 1;
        }
        if (a[field] > b[field]) {
          return -1;
        }
      }
      return 0;
    });
  }

  function sortModels(pstate: PrinterState, allModels: GCodeFileState[]): GCodeFileState[] {
    let models: GCodeFileState[] = [];
    // tslint:disable-next-line
    const reverse = (pstate.sortBy & 1) === 1;
    // tslint:disable-next-line
    switch (pstate.sortBy & 254) {
      case 0:
        models = orderBy(allModels, "name", reverse);
        break;
      case 2:
        models = orderBy(allModels, "created", reverse);
        break;
      case 4:
        models = orderBy(allModels, "printTime", reverse);
        break;
      case 6:
        models = orderBy(allModels, "length", reverse);
        break;
      case 8:
        models = orderBy(allModels, "lines", reverse);
        break;
      case 10:
        models = orderBy(allModels, "filamentTotal", reverse);
        break;
      case 12:
        models = orderBy(allModels, "layer", reverse);
        break;
    }
    if (pstate.modelFilter !== "") {
      const filtered = [];
      for (const v of models) {
        if (v.name.indexOf(pstate.modelFilter) >= 0) {
          filtered.push(v);
        }
      }
      models = filtered;
    }
    if (pstate.activeGroupId !== "*") {
      const filtered = [];
      for (const v of models) {
        if (v.group === pstate.activeGroupId) {
          filtered.push(v);
        }
      }
      models = filtered;
    }
    return models;
  }

  function appendTemperatureEntry(list: TemperatureDataState[], temp: TemperatureDataState): TemperatureDataState[] {
    let start = 0;
    while (list.length > start && list[start].time + 300000 < temp.time) {
      start++;
    }
    return list.slice(start).concat(temp);
  }

  const parts: string[] = action.type.split("/");
  if (parts.length > 1) { // aggregated type with pointer

  } else {
    switch (action.type) {
      case PrinterAction.UPDATE_LOG_STATUS: {
        return Object.assign({}, state, {log: Object.assign({}, state.log, action.payload)});
      }
      case PrinterAction.ADD_LOG_LINE: {
        const log: LogState = Object.assign({}, state.log, {lastLogStart: action.payload.id});
        if (!state.log.logM105) {
          if (action.payload.text.indexOf('M105') !== -1 || testTempResponse.test(action.payload.text)) {
            action.payload.text = '';
          }
        }
        if (!state.log.paused && action.payload.text) {
          const lineText = action.payload.text.replace(/[<>]/ig, function (m) {
            return "&" + (m === ">" ? "g" : "l") + "t;";
          });
          const newline = action.payload.time + ": " + lineText;
          if (log.lastText !== lineText) {
            log.history = log.history.slice(0);
            log.history.push("<div class=\"log" + action.payload.type + "\">" + newline + "</div>");
            log.history.splice(0, log.history.length - log.maxLogLines);
            log.repeat = 1;
            log.lastText = lineText;
          } else { // Repeated lines
            log.repeat++;
            log.history = log.history.slice(0);
            log.history[log.history.length - 1] = "<div class=\"log" + action.payload.type + "\">" + newline + " (" + log.repeat + ")</div>";
          }
        }
        return Object.assign({}, state, {log: log});
      }
      case PrinterAction.SET_CONFIG: {
        const extra = {config: action.payload.config};
        if (extra.config.extruders.length > state.activeExtruderId) {
          extra["retractSpeed"] = extra.config.extruders[state.activeExtruderId].retractSpeed;
          extra["extrudeSpeed"] = extra.config.extruders[state.activeExtruderId].extrudeSpeed;
        }
        return Object.assign({}, state, extra);
      }
      case PrinterAction.SET_WEBCAM_CONFIG: {
        const extra = {config: Object.assign({}, state.config, {webcams: action.payload.webcams})};
        return Object.assign({}, state, extra);
      }
      case PrinterAction.RESET_IMAGE_REFERENCE:
        return Object.assign({}, state, {imageUrls: {}});
      case PrinterAction.CLEAR_EEPROM:
        return Object.assign({}, state, {eeprom: [], eepromErrors: false});
      case PrinterAction.COPY_EEPROM_VALUE_TO_ORIG: {
        const epr = RSUtils.copyArray(state.eeprom);
        for (const v of epr) {
          v.valueOrig = v.value;
          v.class = "secondary outline";
        }
        return Object.assign({}, state, {eeprom: epr, eepromErrors: false});
      }
      case PrinterAction.SET_NEW_EEPROM_VALUE: {
        const epr = RSUtils.copyArray(state.eeprom);
        for (const v of epr) {
          if (v.pos === action.payload.pos) {
            v.value = action.payload.value;
          }
        }
        let okAll = true;
        for (const v of epr) {
          const r = /^-?\d+$/;
          const f = /^-?\d+\.?\d*$/;
          let val;
          let ok = true;
          if (v.type === "0") { // byte
            if (r.exec(v.value) === null) {
              ok = false;
            } else {
              val = parseInt(v.value, 10);
              ok = (val > -128 && val < 256);
            }
          }
          if (v.type === "1") { // word
            if (r.exec(v.value) === null) {
              ok = false;
            } else {
              val = parseInt(v.value, 10);
              ok = (val > -32768 && val < 65536);
            }
          }
          if (v.type === "2") { // long
            if (r.exec(v.value) === null) {
              ok = false;
            } else {
              val = parseInt(v.value, 10);
              ok = (val > -2147483648 && val < 4294967296);
            }
          }
          if (v.type === "3") { // float
            if (f.exec(v.value) === null) {
              ok = false;
            }
          }
          if (ok) {
            if (v.value === v.valueOrig) {
              v.class = "secondary outline";
            } else {
              v.class = "success";
            }
          } else {
            v.class = "error";
          }
          okAll = okAll && ok;
        }
        return Object.assign({}, state, {eeprom: epr, eepromErrors: !okAll});
      }
      case PrinterAction.APPEND_EEPROM: {
        const epr = RSUtils.copyArray(state.eeprom);
        for (const pl of action.payload) {
          const text = pl.text;
          let textNU = "";
          let unit = "";
          const p = text.indexOf("[");
          if (p >= 0) {
            textNU = text.substr(0, p);
          } else {
            textNU = text;
          }
          if (p >= 0) {
            const p2 = text.indexOf("]", p);
            if (p2 >= 0) {
              unit = text.substring(p + 1, p2).replace("^2", "²").replace("^3", "³");
            }
          }
          const x = Object.assign({}, pl, {textNU: textNU, unit: unit, class: "secondary outline"});
          epr.push(x);
        }
        return Object.assign({}, state, {eeprom: epr});
      }
      case PrinterAction.SET_RECOVER:
        return Object.assign({}, state, {recover: action.payload});
      case PrinterAction.UPDATE_SCRIPT: {
        const extra = {};
        extra[action.payload.name] = action.payload.script;
        return Object.assign({}, state, {scripts: Object.assign({}, state.scripts, extra)});
      }
      case PrinterAction.SET_GROUPS: {
        const newGroups: GroupState[] = [];
        const baseGroups: GroupState[] = [];
        const baseGroups2: GroupState[] = [];
        for (const x in action.payload.groups) {
          if (action.payload.groups.hasOwnProperty(x)) {
            const g = action.payload.groups[x];
            if (g.id === "#" || g.id === "*") {
              baseGroups.push(g);
              if (g.id === "#") {
                baseGroups2.push(g);
              }
            } else {
              newGroups.push(action.payload.groups[x]);
            }
          }
        }
        return Object.assign({}, state, {
          groups: action.payload.groups,
          activeGroupId: action.payload.activeGroup,
          groupsSorted: baseGroups.concat(orderBy(newGroups, "name", true)),
          groupsSortedNoAll: baseGroups2.concat(orderBy(newGroups, "name", true))
        });
      }
      case PrinterAction.SET_DIALOGS: {
        return Object.assign({}, state, {dialogs: action.payload});
      }
      case PrinterAction.UPDATE_STATE: {
        const newState = Object.assign({}, state, action.payload.state);
        if (state.activeGroupId !== newState.activeGroupId) {
          newState.modelsFiltered = sortModels(newState, newState.allModels);
        }
        return newState;
      }
      case PrinterAction.UPDATE_STATUS:
        return Object.assign({}, state, action.payload.status);
      case PrinterAction.SET_IMG_URL: {
        const a: { [key: string]: string } = {};
        a[action.payload.key] = action.payload.url;
        return Object.assign({}, state, {imageUrls: Object.assign({}, state.imageUrls, a)});
      }
      case PrinterAction.UPDATE_IMG_URLS:
        return Object.assign({}, state, {imageUrls: Object.assign({}, state.imageUrls, action.payload)});
      case PrinterAction.SET_MODELS: {
        return Object.assign({}, state, {
          allModels: action.payload,
          modelsFiltered: sortModels(state, action.payload)
        });
      }
      case PrinterAction.SET_SORT_BY: {
        return Object.assign({}, state, {sortBy: action.payload});
      }
      case PrinterAction.SORT_MODELS: {
        return Object.assign({}, state, {modelsFiltered: sortModels(state, state.allModels)});
      }
      case PrinterAction.SET_PRINTQUEUE:
        return Object.assign({}, state, {queue: action.payload});
      case PrinterAction.ADD_BED_TEMPERATURE: {
        const newBed: ToolState[] = RSUtils.copyArray(state.heatedBeds);
        newBed[0].tempHistory = appendTemperatureEntry(newBed[action.payload.id].tempHistory, action.payload.temp);
        return Object.assign({}, state, {heatedBeds: newBed});
      }
      case PrinterAction.ADD_CHAMBER_TEMPERATURE: {
        const newChamber: ToolState[] = RSUtils.copyArray(state.heatedChambers);
        newChamber[0].tempHistory = appendTemperatureEntry(newChamber[action.payload.id].tempHistory, action.payload.temp);
        return Object.assign({}, state, {heatedChambers: newChamber});
      }
      case PrinterAction.ADD_EXTRUDER_TEMPERATURE: {
        const newExtruders: ToolState[] = RSUtils.copyArray(state.extruders);
        newExtruders[action.payload.extruderId].tempHistory = appendTemperatureEntry(newExtruders[action.payload.extruderId].tempHistory, action.payload.temp);
        return Object.assign({}, state, {extruders: newExtruders});
      }
      case PrinterAction.UPDATE_GCODE: {
        const g: GCodeFileState = action.payload;
        let idx = -1;
        const newState: PrinterState = Object.assign({}, state);
        if (g.list === "models") {
          for (let i = 0; i < newState.allModels.length; i++) {
            if (newState.allModels[i].id === g.id) {
              idx = i;
              break;
            }
          }
          if (idx !== -1) {
            newState.allModels = newState.allModels.slice(0, idx).concat([g], newState.allModels.slice(idx + 1));
            newState.modelsFiltered = sortModels(newState, newState.allModels);
          }
        } else {
          for (let i = 0; i < newState.queue.length; i++) {
            if (newState.queue[i].id === g.id) {
              idx = i;
              break;
            }
          }
          if (idx !== -1) {
            newState.queue = newState.queue.slice(0, idx).concat([g], newState.queue.slice(idx + 1));
          }
        }
        return newState;
      }
      default:
        return state;
    }
  }
}
